#![allow(clippy::missing_panics_doc)]
use std::io::BufReader;

use crate::identification::{test_format, PLGSData, PLGSVersion};

#[test]
fn plgs_fragment() {
    match test_format::<PLGSData>(
        BufReader::new(DATA_FRAGMENT.as_bytes()),
        None,
        true,
        false,
        Some(PLGSVersion::V3_0),
    ) {
        Ok(n) => assert_eq!(n, 22),
        Err(e) => {
            println!("{e}");
            panic!("Failed identified peptides test");
        }
    }
}

#[test]
fn plgs_peptide() {
    match test_format::<PLGSData>(
        BufReader::new(DATA_PEPTIDE.as_bytes()),
        None,
        true,
        false,
        Some(PLGSVersion::V3_0),
    ) {
        Ok(n) => assert_eq!(n, 21),
        Err(e) => {
            println!("{e}");
            panic!("Failed identified peptides test");
        }
    }
}

const DATA_FRAGMENT: &str = r"protein.key,protein.Entry,protein.Accession,protein.Description,protein.dataBaseType,protein.score,protein.falsePositiveRate,protein.avgMass,protein.MatchedProducts,protein.matchedPeptides,protein.digestPeps,protein.seqCover(%),protein.MatchedPeptideIntenSum,protein.top3MatchedPeptideIntenSum,protein.MatchedProductIntenSum,protein.fmolOnColumn,protein.ngramOnColumn,protein.AutoCurate,protein.Key_ForHomologs,peptide.Rank,peptide.Pass,peptide.matchType,peptide.modification,peptide.mhp,peptide.seq,peptide.OriginatingSeq,peptide.seqStart,peptide.seqLength,peptide.pI,peptide.componentID,peptide.MatchedProducts,peptide.UniqueProducts,peptide.ConsectiveMatchedProducts,peptide.ComplementaryMatchedProducts,peptide.rawScore,peptide.score,peptide.(X)-P Bond,peptide.MatchedProductsSumInten,peptide.MatchedProductsTheoretical,peptide.MatchedProductsString,peptide.ModelRT,peptide.Volume,peptide.CSA,peptide.ModelDrift,peptide.RelIntensity,peptide.AutoCurate,precursor.leID,precursor.mhp,precursor.mhpCal,precursor.retT,precursor.inten,precursor.calcInten,precursor.charge,precursor.z,precursor.mz,precursor.fwhm,precursor.liftOffRT,precursor.infUpRT,precursor.infDownRT,precursor.touchDownRT,prec.rmsFWHMDelta,protein.SumForTotalProteins,peptide.SumForTotalPeps,fragment.mhp,fragment.fragmentType,fragment.fragInd,Neutral.LossType,fragment.str,fragment.seq,fragment.fragSite,product.rank,product.isLinked,product.heID,product.mhp,product.mhpCal,product.m_z,product.retT,product.inten,product.charge,product.z,product.fwhm,product.liftOffRT,product.infUpRT,product.infDownRT,product.touchDownRT,fragmentProduct.deltaMhpPPM,precursorProduct.deltaRetT,peptidePrecursor.deltaMhpPPM
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,1,1,363.2027,b,3,None,b3,KYA,AP,1,False,37990,363.2104,363.2104,363.2104,4.5221,15601,1.00,1,0.09049957,4.391343,4.486395,4.556062,4.681931,21.3220,0.0018,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,460.2554,b,4,None,b4,KYAP,PL,2,False,37940,460.2597,460.2597,460.2597,4.5218,16673,1.00,1,0.09416644,4.381315,4.485775,4.556681,4.688846,9.1465,0.0021,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,573.3395,b,5,None,b5,KYAPL,LY,3,False,38042,573.3387,573.3387,573.3387,4.5226,30837,1.00,1,0.0951906,4.378022,4.486072,4.556904,4.685687,-1.4629,0.0012,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,736.4028,b,6,None,b6,KYAPLY,YA,4,False,37896,736.3921,736.3921,736.3921,4.5213,12254,1.06,1,0.1096353,4.407772,4.48524,4.555354,4.692621,-14.5504,0.0026,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1007.5197,b,9,None,b9,KYAPLYAAE,EA,5,False,38130,1007.4923,1007.4923,1007.4923,4.5238,842,1.00,1,0.06422713,4.479074,4.495646,4.558017,4.586696,-27.2020,0.0001,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1060.5462,b,10,H2O,b10°,KYAPLYAAEA,AK,6,False,37880,1060.5504,1060.5504,1060.5504,4.5211,6761,1.00,1,0.08934911,4.421024,4.484574,4.556566,4.642402,3.9076,0.0028,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1188.6412,b,11,H2O,b11°,KYAPLYAAEAK,KR,7,False,37821,1188.6355,1188.6355,1188.6355,4.5205,5579,1.34,1,0.09466767,4.45426,4.486207,4.555106,4.617718,-4.7694,0.0033,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1362.7528,b,12,None,b12,KYAPLYAAEAKR,RV,8,False,37462,1362.7406,1362.7406,681.8740,4.5161,248,2.00,2,0.06359407,4.451601,4.48413,4.536582,4.556263,-8.9672,0.0077,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1344.7423,b,12,H2O,b12°,KYAPLYAAEAKR,RV,9,False,37522,1344.7417,1344.7417,672.8745,4.5170,435,2.00,2,0.08006313,4.469313,4.491289,4.552901,4.586787,-0.4314,0.0069,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,2066.1433,b,18,None,b18,KYAPLYAAEAKRVFSLEK,KK,10,False,38316,2066.1225,2066.1225,689.3790,4.5273,10405,2.50,3,0.08530093,4.440483,4.486742,4.557299,4.61864,-10.0770,-0.0034,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1639.9166,y,14,None,y14,YAAEAKRVFSLEKK,LY,11,False,38020,1639.9201,1639.9201,547.3115,4.5224,84112,2.62,3,0.1344313,4.321313,4.485588,4.557793,4.694162,2.1069,0.0015,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1621.9061,y,14,H2O,y14°,YAAEAKRVFSLEKK,LY,12,False,38168,1621.9062,1621.9062,541.3069,4.5242,6020,2.78,3,0.09690656,4.369566,4.486159,4.558846,4.617004,0.0736,-0.0004,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1753.0007,y,15,None,y15,LYAAEAKRVFSLEKK,PL,13,False,37660,1753.0160,1753.0160,439.0095,4.5186,472,4.00,4,0.08610346,4.436711,4.478846,4.552921,4.626598,8.7302,0.0053,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1850.0534,y,16,None,y16,PLYAAEAKRVFSLEKK,AP,14,False,38014,1850.0081,1850.0081,925.5077,4.5223,49183,2.25,2,0.08787873,4.408198,4.485925,4.55619,4.617931,-24.4940,0.0015,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1832.0429,y,16,H2O,y16°,PLYAAEAKRVFSLEKK,AP,15,False,38076,1831.9928,1831.9928,916.5000,4.5230,3857,2.18,2,0.1012462,4.454793,4.483958,4.575658,4.632731,-27.3262,0.0009,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1921.0906,y,17,None,y17,APLYAAEAKRVFSLEKK,YA,16,False,38059,1921.0387,1921.0387,961.0230,4.5228,26620,2.15,2,0.1167802,4.417428,4.486533,4.558119,4.633074,-27.0091,0.0010,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,0,0,1903.0800,y,17,H2O,y17°,APLYAAEAKRVFSLEKK,YA,17,False,38005,1903.0284,1903.0284,952.0178,4.5223,1577,2.18,2,0.08922738,4.424727,4.486602,4.556171,4.624454,-27.1119,0.0016,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,2,Pass1,PepFrag1,None,1245.6296,MSNYIQFKSK,MSNYIQFKSK,78,10,10.18,13,13,13,9,5,6831.2570,8.7879,,163857,12.0,b3b4b4*b4°b5b5°b6b7b7*b9b10b10*y3y3°y4y5y5*y5°y6y7y7*y7°y9y9*,4.2470,1150,132.706296038843,-0.512986876092312,0.0210,Green,3147,1245.6235,1245.6235,4.2470,1969575,1969575.00,1.94,2,623.3154,0.07912405,4.160105,4.213992,4.280109,4.350046,1.6046E-001,0,1,333.1227,b,3,None,b3,MSN,NY,1,False,27376,333.1310,333.1310,333.1310,4.2464,4136,1.00,1,0.08790599,4.147377,4.214796,4.278599,4.393572,24.9590,0.0006,-4.9410
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,2,Pass1,PepFrag1,None,1245.6296,MSNYIQFKSK,MSNYIQFKSK,78,10,10.18,13,13,13,9,5,6831.2570,8.7879,,163857,12.0,b3b4b4*b4°b5b5°b6b7b7*b9b10b10*y3y3°y4y5y5*y5°y6y7y7*y7°y9y9*,4.2470,1150,132.706296038843,-0.512986876092312,0.0210,Green,3147,1245.6235,1245.6235,4.2470,1969575,1969575.00,1.94,2,623.3154,0.07912405,4.160105,4.213992,4.280109,4.350046,1.6046E-001,0,0,496.1860,b,4,None,b4,MSNY,YI,2,False,27480,496.1884,496.1884,496.1884,4.2479,3486,1.00,1,0.06549857,4.153439,4.215396,4.281171,4.345059,4.8287,-0.0009,-4.9410
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,4178,Pass2,VarMod,Oxidation+M(2);Oxidation+M(20),2577.0975,IMDDLCQNPDGKDVSLFCQMVSS,IMDDLCQNPDGKDVSLFCQMVSS,144,23,3.43,852,2,2,1,0,143.6626,7.1107,Missing,2416,1.8,b11b12b13°b17°y12°y13°,8.6253,2317,211.694864879392,-1.06132448738702,0.0000,Yellow,66610,2577.1078,2577.1078,8.6253,1009,1009.00,2.00,2,1289.0575,0.06192559,8.578293,8.60381,8.687894,8.704979,2.6230E-001,0,0,1341.6177,y,12,H2O,y12°,KDVSLFCQMVSS,GK,5,False,245492,1341.6187,1341.6187,1341.6187,8.6212,684,1.00,1,0.06573262,8.573329,8.590278,8.643667,8.698474,0.7086,0.0041,3.9839
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,4179,Pass2,VarMod,Oxidation+M(28);Oxidation+M(35),4614.2827,LASTNAGSINAPTVSDSRALARRFHFDMNIEVISMYSQNGKI,LASTNAGSINAPTVSDSRALARRFHFDMNIEVISMYSQNGKI,189,42,9.71,962,5,5,1,1,213.5992,7.2830,Missing,1202,2.2,b15b33y8y9y15*y37,7.8136,4244,316.91530814765,-1.5534276440232,0.0000,Yellow,53735,4614.2897,4614.2897,7.8136,1001,1001.00,4.54,4,1154.3279,0.08579935,7.721161,7.744169,7.829408,7.849741,4.2991E-001,0,0,1741.8135,y,15,NH3,y15*,MNIEVISMYSQNGKI,DM,5,False,202688,1741.8278,1741.8278,1741.8278,7.8228,99,1.00,1,0.04574742,7.774236,7.796287,7.84793,7.874473,8.2307,-0.0092,1.5090
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,429,Pass2,PepFrag2,None,2359.0626,APSQSDQEQLFSNVQYFAHY,APSQSDQEQLFSNVQYFAHY,37,20,4.06,2798,0,0,0,0,140.6496,7.1015,Missing,0,6.5,,4.6156,2094,197.884032748452,-0.849627901687047,0.0004,Yellow,5144,2359.1177,2359.1177,4.6156,40089,40089.00,3.42,3,787.0441,0.0824611,4.492433,4.580017,4.655515,4.862597,0.0000E+000,0,1,,,,,,,,-1,,,,,,,,,,,,,,,,,23.3782";

const DATA_PEPTIDE: &str = r"protein.key,protein.Entry,protein.Accession,protein.Description,protein.dataBaseType,protein.score,protein.falsePositiveRate,protein.avgMass,protein.MatchedProducts,protein.matchedPeptides,protein.digestPeps,protein.seqCover(%),protein.MatchedPeptideIntenSum,protein.top3MatchedPeptideIntenSum,protein.MatchedProductIntenSum,protein.fmolOnColumn,protein.ngramOnColumn,protein.AutoCurate,protein.Key_ForHomologs,protein.SumForTotalProteins,peptide.Rank,peptide.Pass,peptide.matchType,peptide.modification,peptide.mhp,peptide.seq,peptide.OriginatingSeq,peptide.seqStart,peptide.seqLength,peptide.pI,peptide.componentID,peptide.MatchedProducts,peptide.UniqueProducts,peptide.ConsectiveMatchedProducts,peptide.ComplementaryMatchedProducts,peptide.rawScore,peptide.score,peptide.(X)-P Bond,peptide.MatchedProductsSumInten,peptide.MatchedProductsTheoretical,peptide.MatchedProductsString,peptide.ModelRT,peptide.Volume,peptide.CSA,peptide.ModelDrift,peptide.RelIntensity,peptide.AutoCurate,precursor.leID,precursor.mhp,precursor.mhpCal,precursor.retT,precursor.inten,precursor.calcInten,precursor.charge,precursor.z,precursor.mz,precursor.fwhm,precursor.liftOffRT,precursor.infUpRT,precursor.infDownRT,precursor.touchDownRT,prec.rmsFWHMDelta,peptidePrecursor.deltaMhpPPM
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,1,1,Pass1,PepFrag1,None,2212.2489,KYAPLYAAEAKRVFSLEKK,KYAPLYAAEAKRVFSLEKK,59,19,10.11,19,11,11,6,3,3051.0050,8.4378,Identified,271476,30.0,b3b4b5b6b9b10°b11°b12b12°b18y14y14°y15y16y16°y17y17°,4.5239,2107,198.702191926964,-0.744767652397498,0.0454,Green,4582,2212.2540,2212.2540,4.5239,4264229,4264229.00,3.38,4,553.8190,0.08156413,4.340279,4.486571,4.558985,4.888723,2.4631E-001,2.3163
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,2,Pass1,PepFrag1,None,1245.6296,MSNYIQFKSK,MSNYIQFKSK,78,10,10.18,13,13,13,9,5,6831.2570,8.7879,,163857,12.0,b3b4b4*b4°b5b5°b6b7b7*b9b10b10*y3y3°y4y5y5*y5°y6y7y7*y7°y9y9*,4.2470,1150,132.706296038843,-0.512986876092312,0.0210,Green,3147,1245.6235,1245.6235,4.2470,1969575,1969575.00,1.94,2,623.3154,0.07912405,4.160105,4.213992,4.280109,4.350046,1.6046E-001,-4.9410
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,3,Pass1,PepFrag1,None,1849.9775,CRIEPVCLLLHGSPGAGK,CRIEPVCLLLHGSPGAGK,88,18,8.30,28,20,20,14,6,13028.4700,9.0682,Identified,1089488,23.9,b3b4b4°b5b6b7b8b9b12b18y4y5y6y6°y7y7°y8y8°y9y10y12y13y14y18,4.9427,1747,175.369839368313,-0.666278437248658,0.0167,Green,7242,1849.9703,1849.9703,4.9427,1563412,1563412.00,2.72,3,617.3283,0.1074543,4.793483,4.908295,4.981444,5.361441,1.8692E-001,-3.9185
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,4,Pass1,PepFrag1,None,1130.6528,SVATNLIGRSL,SVATNLIGRSL,106,11,11.05,37,10,10,6,3,6734.3700,8.7817,,272372,14.0,b4b4°b5b5*b5°b6b6°b8*b10b10°y3y4°y5y6y7y7*y8y8°y9,5.1605,1075,126.871851282107,-0.465636042558873,0.0101,Green,9377,1130.6545,1130.6545,5.1605,946465,946465.00,1.66,2,565.8309,0.09022491,5.046968,5.129726,5.193929,5.470443,2.8487E-001,1.5378
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,5,Pass1,PepFrag1,None,1865.9504,GPGGGGSLPEVREKHEFL,GPGGGGSLPEVREKHEFL,0,18,5.40,15,14,14,8,0,4734.6190,8.6286,Identified,514290,18.5,b5b6b7b7°b8°b11°b15b16b17b17°b18b18°y4y4°y5y7°y8y10y10°y16y17y17°y18,4.4153,1709,172.817483176688,-0.672031161661474,0.0082,Green,3784,1865.9427,1865.9427,4.4153,773999,773999.00,2.69,3,622.6524,0.08230924,4.268733,4.383899,4.447145,4.656919,1.5167E-001,-4.1302
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,6,Pass1,PepFrag1,None,1827.0163,KYAPLYAAEAKRVFSL,KYAPLYAAEAKRVFSL,59,16,9.94,38,8,8,4,2,1165.6320,8.0199,Identified,440993,16.1,b4b5b11°b14y11y12y13y15y16,5.1615,1728,174.096,-0.658008853632055,0.0064,Green,9396,1827.0087,1827.0087,5.1615,600100,600100.00,2.60,3,609.6744,0.104474,4.985098,5.12932,5.195953,5.503819,1.7104E-001,-4.2002
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,7,Pass1,PepFrag1,None,2177.0193,EMFREYNHRHSVGTTLEA,EMFREYNHRHSVGTTLEA,279,18,6.04,679,3,3,0,0,356.2221,7.5051,,292286,16.4,b10*b11b16°y16y18,4.1686,1958,189.220459379412,-0.896560496635691,0.0060,Green,2252,2177.0252,2177.0252,4.1686,559348,559348.00,3.00,2,1089.0162,0.07428372,4.05398,4.138342,4.200608,4.401914,3.0498E-001,2.7281
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,8,Pass1,PepFrag1,None,1626.9002,KYAPLYAAEAKRVF,KYAPLYAAEAKRVF,59,14,9.94,21,9,9,7,0,3190.8600,8.4573,Identified,186507,11.7,b13°y4y5y6y6°y7y8y8°y11y12y13y14,4.6216,1531,160.599095378901,-0.585936065939429,0.0030,Green,5203,1626.9028,1626.9028,4.6216,281289,281289.00,2.56,3,542.9725,0.1072639,4.438976,4.588928,4.656482,4.883434,2.9342E-001,1.5934
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,9,Pass1,PepFrag1,None,2989.5869,FAHYCRKYAPLYAAEAKRVFSLEKK,FAHYCRKYAPLYAAEAKRVFSLEKK,53,25,9.97,279,8,8,2,0,1046.0970,7.9729,Identified,148781,14.0,b3b7b9b10b11b13b24y9°y22°y25,4.4770,2802,240.291488761949,-1.07671432588489,0.0028,Green,3968,2989.5767,2989.5767,4.4770,262181,262181.00,4.48,3,997.1971,0.1031726,4.390141,4.441827,4.512157,4.73572,2.3583E-001,-3.4283
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,10,Pass1,PepFrag1,None,1752.8664,GPGGGGSLPEVREKHEF,GPGGGGSLPEVREKHEF,0,17,5.40,6,12,12,5,2,9004.0270,8.9078,Identified,144188,11.7,b7b7°b8b8°b10b17b17°y3y4y7y7°y9y10°y14y15y16y17,3.9998,1585,164.353571210361,-0.721881875364834,0.0025,Green,1906,1752.8642,1752.8642,3.9998,230905,230905.00,2.63,2,876.9357,0.07537718,3.912741,3.969634,4.031596,4.197194,2.2898E-001,-1.2411
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,11,Pass1,PepFrag1,None,2215.1538,ASTNAGSINAPTVSDSRALARR,ASTNAGSINAPTVSDSRALARR,190,22,12.10,2,11,11,4,2,6387.8890,8.7587,Identified,22622,12.5,b4b5b5*b7*b14*b15°b22b22°y9y9°y11y12y14y17y17*y18y20y20°y21,3.8132,2031,193.894839181873,-0.797798450068489,0.0023,Green,1191,2215.1569,2215.1569,3.8132,218657,218657.00,3.05,3,739.0571,0.09300872,3.665533,3.780923,3.843863,4.000083,2.8454E-001,1.3951
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,12,Pass1,PepFrag1,None,1662.7846,FSNVQYFAHYCRK,FSNVQYFAHYCRK,47,13,9.56,16,8,8,3,0,3488.3080,8.4960,,115123,10.1,b1b12°y4y5y6y8y9*y10y11y13,4.5004,1494,158.001071622971,-0.684783536352153,0.0022,Green,4075,1662.7863,1662.7863,4.5004,205222,205222.00,2.68,2,831.8968,0.1021017,4.404638,4.468365,4.535386,4.728908,1.9926E-001,1.0475
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,13,Pass1,PepFrag1,None,1121.5810,NHRHSVGTTL,NHRHSVGTTL,285,10,11.05,1,6,6,3,1,1190.2590,8.0290,,12683,7.5,b3b3*b4*b5°b6b7b7*b8b8°b9b9°b10°y4°y7y7°,2.7321,1016,122.186185295467,-0.461900021950981,0.0012,Green,427,1121.5830,1121.5830,2.7321,115521,115521.00,1.77,2,561.2951,0.0641652,2.647308,2.701649,2.761749,3.050935,1.2680E-001,1.7143
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,14,Pass1,PepFrag1,None,1639.9166,YAAEAKRVFSLEKK,YAAEAKRVFSLEKK,64,14,10.04,304,13,13,7,3,4268.2580,8.5836,,162286,8.4,b4b7b8b8°b11°b13b14°y3y3°y4y6y7y9y10y11y11°y12y12°y13y13°,4.5231,1550,161.925072300479,-0.590623977674424,0.0012,Green,4562,1639.9199,1639.9199,4.5231,112712,112712.00,3.03,3,547.3115,0.09207654,4.344988,4.485009,4.558352,4.664087,1.1639E-001,1.9856
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,15,Pass1,PepFrag1,None,2623.2392,SLDMLVTEMFREYNHRHSVGTT,SLDMLVTEMFREYNHRHSVGTT,272,22,6.03,50,2,2,0,0,163.0623,7.1657,,75564,9.3,b5b13°y9*y22,5.5612,2392,216.238863959794,-1.08032696428916,0.0010,Yellow,13108,2623.2281,2623.2281,5.5612,96559,96559.00,2.25,2,1312.1177,0.1234672,5.422443,5.523505,5.597524,5.715682,4.2285E-001,-4.2109
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,16,Pass1,PepFrag1,None,1681.9133,QFIDRRTQVRYSL,QFIDRRTQVRYSL,261,13,11.04,23,7,7,2,2,1814.6700,8.2122,,29866,6.5,b3b3*b4b4°b7*b10*b12*y4y6y6*y9y9*y10y12,4.6822,1558,162.481755656956,-0.605749274078947,0.0007,Green,5582,1681.9136,1681.9136,4.6822,62117,62117.00,2.81,3,561.3094,0.1265317,4.544012,4.646751,4.716959,4.944796,3.7986E-001,0.2166
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,17,Pass1,PepFrag1,None,2703.4651,AAEAKRVFSLEKKMSNYIQFKSK,AAEAKRVFSLEKKMSNYIQFKSK,65,23,10.46,308,7,7,0,0,1490.9180,8.1268,,31546,7.9,b14b17b23y3y8y16°y21y23,4.6061,2559,226.189873717998,50.4759494871992,0.0006,Green,5064,2703.4676,2703.4676,4.6061,59248,59248.00,4.17,5,541.4993,0.1255453,4.487636,4.569167,4.645281,4.76484,4.8483E-001,0.9351
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,18,Pass1,PepFrag1,None,1175.5361,CQNPDGKDVSL,CQNPDGKDVSL,149,11,3.88,459,4,4,0,1,644.7015,7.7627,Identified,3005,5.6,b3b6y6y8,4.1284,1053,125.134928072436,-0.484120310431522,0.0005,Green,2155,1175.5333,1175.5333,4.1284,48354,48354.00,1.59,2,588.2703,0.07837489,4.025074,4.097431,4.160083,4.373043,1.9299E-001,-2.3904
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,19,Pass1,PepFrag1,None,2086.1266,CRKYAPLYAAEAKRVFSL,CRKYAPLYAAEAKRVFSL,57,18,9.95,285,5,5,0,1,1081.5050,7.9874,Identified,31397,6.7,b5b7y10y13y18,4.9437,1962,189.478077481372,-0.702307803184673,0.0005,Green,7254,2086.1349,2086.1349,4.9437,47678,47678.00,3.09,4,522.2892,0.1039877,4.874422,4.908285,4.980547,5.032216,2.4764E-001,3.9544
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,4223,Pass2,VarMod,Oxidation+M(*);Oxidation+M(*),5016.2767,MNIEVISMYSQNGKINMPMSVKTCDDECCPVNFKKCCPLVCGKAI,MNIEVISMYSQNGKINMPMSVKTCDDECCPVNFKKCCPLVCGKAI,216,45,7.62,1067,1,1,0,0,68.3802,6.7883,Missing,1638,2.1,b14*y19*y40*y41,8.4544,4646,336.624538617983,94.6498154471931,0.0000,Red,63622,5016.2548,5016.2548,8.4544,758,758.00,4.54,5,1004.0568,0.05439995,8.418533,8.434888,8.489266,8.510838,2.5204E-001,-4.3559
0,Accession,Unknown,mono2C_36_cleaved Unknown Entry,Regular,89816.0800,0.00,33499.5799,7491,4224,5901,100.00,9.382709E+07,1.364381E+07,4.463762E+07,,,Green,,0,4224,Pass2,VarMod,Oxidation+M(30);Oxidation+M(39);Oxidation+M(41),4704.2425,AGSINAPTVSDSRALARRFHFDMNIEVISMYSQNGKINMPMS,AGSINAPTVSDSRALARRFHFDMNIEVISMYSQNGKINMPMS,194,42,9.71,978,3,3,0,0,277.3231,7.3963,Missing,1367,2.0,b18*y6°y11°y15y15*y17y18°y23,8.0475,4298,319.597893590087,-1.69425591281921,0.0000,Yellow,57718,4704.2361,4704.2361,8.0475,750,750.00,3.00,3,1568.7502,0.08344793,7.95975,7.987086,8.086679,8.112692,4.6232E-001,-1.3536";
